@@ -43,6 +43,9 @@
 
 import os
 import sys
+import codecs
+
+from enchant.errors import *
 
 # Attempt to access local language information
 try:
@@ -97,13 +100,22 @@ def raw_unicode(raw):
       s = raw_unicode(r"hello\u2149")  # works everywhere!
 
     """
-    return raw.encode("ascii").decode("unicode-escape")
+    return raw.encode("utf8").decode("unicode-escape")
 
 
+def raw_bytes(raw):
+    """Make a bytes object out of a raw string.
+
+    This is analogous to raw_unicode, but processes byte escape characters
+    to produce a bytes object.
+    """
+    return codecs.escape_decode(raw)[0]
+        
+
 class EnchantStr(str):
     """String subclass for interfacing with enchant C library.
 
-    This class encapsulate the logic for interfacing between python native
+    This class encapsulates the logic for interfacing between python native
     string/unicode objects and the underlying enchant library, which expects
     all strings to be UTF-8 character arrays.  It is a subclass of the
     default string class 'str' - on Python 2.x that makes it an ascii string,
@@ -117,7 +129,7 @@ class EnchantStr(str):
     This allows us to following the common Python 2.x idiom of returning
     unicode when unicode is passed in, and byte strings otherwise.  It also
     lets the interface be upwards-compatible with Python 3, in which string
-    objects will be unicode by default.
+    objects are unicode by default.
     """
 
     def __new__(cls,value):
@@ -127,13 +139,13 @@ class EnchantStr(str):
         simply passes it along to the default string constructor.
         """
         if type(value) is unicode:
-          was_unicode = True
-          if str is not unicode:
-            value = value.encode("utf-8")
+            was_unicode = True
+            if str is not unicode:
+                value = value.encode("utf-8")
         else:
-          was_unicode = False
-          if str is not bytes:
-            raise RuntimeError("Don't pass bytestrings to pyenchant")
+            was_unicode = False
+            if str is not bytes:
+                raise Error("Don't pass bytestrings to pyenchant")
         self = str.__new__(cls,value)
         self._was_unicode = was_unicode
         return self
@@ -149,15 +161,42 @@ class EnchantStr(str):
         """Decode a string returned by the enchant C library."""
         if self._was_unicode:
           if str is unicode:
-            # TODO: why does ctypes convert c_char_p to str(),
-            #       rather than to bytes()?
-            return value.encode().decode("utf-8")
+            # On some python3 versions, ctypes converts c_char_p
+            # to str() rather than bytes()   
+            if isinstance(value,str):
+                value = value.encode()
+            return value.decode("utf-8")
           else:
             return value.decode("utf-8")
         else:
           return value
 
 
+def printf(values,sep=" ",end="\n",file=None):
+    """Compatability wrapper from print statement/function.
+
+    This function is a simple Python2/Python3 compatability wrapper
+    for printing to stdout.
+    """
+    if file is None:
+        file = sys.stdout
+    file.write(sep.join(map(str,values)))
+    file.write(end)
+
+
+try:
+    next = next
+except NameError:
+    def next(iter):
+        """Compatability wrapper for advancing an iterator."""
+        return iter.next()
+
+try:
+    xrange = xrange
+except NameError:
+    xrange = range
+
+
 def get_default_language(default=None):
     """Determine the user's default language, if possible.
     
@@ -169,7 +208,7 @@ def get_default_language(default=None):
           that language is used
         * if a default locale is available, that language is used
         * if the keyword argument <default> is given, it is used
-        * None
+        * if nothing else works, None is returned
         
     Note that determining the user's language is in general only
     possible if they have set the necessary environment variables
@@ -183,7 +222,7 @@ def get_default_language(default=None):
             if tag is None:
                 raise Error("No default language available")
         return tag
-    except:
+    except Exception:
         pass
     return default
 get_default_language._DOC_ERRORS = ["LC"]
@@ -207,8 +246,14 @@ def get_resource_filename(resname):
             return path
     else:
         import pkg_resources
-        return pkg_resources.resource_filename("enchant",resname)
-    raise RuntimeError("Could not locate resource '%s'" % (resname,))
+        try:
+            path = os.path.abspath(pkg_resources.resource_filename("enchant",resname))
+        except KeyError:
+            pass
+        else:
+            if os.path.exists(path):
+                return path
+    raise Error("Could not locate resource '%s'" % (resname,))
 
 
 def win32_data_files():
@@ -224,7 +269,10 @@ def win32_data_files():
     which we ship our own supporting data files)
     """
     #  Include the main enchant DLL
-    libEnchant = get_resource_filename("libenchant.dll")
+    try:
+        libEnchant = get_resource_filename("libenchant.dll")
+    except Error:
+        libEnchant = get_resource_filename("libenchant-1.dll")
     mainDir = os.path.dirname(libEnchant)
     dataFiles = [('',[libEnchant])]
     #  And some specific supporting DLLs
@@ -250,8 +298,4 @@ def win32_data_files():
     return dataFiles
 win32_data_files._DOC_ERRORS = ["py","py","exe"]
 
-# Make enchant.Error available
-# Done at bottom of file to avoid circular imports
-from enchant import Error
-
 
